'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');
require('../_util/hooks/index.js');
var useDraggable = require('../_util/Dom/useDraggable.js');
var useMergedState = require('../_util/hooks/useMergedState.js');

const sticks = Object.freeze(["tl", "tc", "tr", "cl", "cr", "bl", "bc", "br"]);
const prefixCls = "x-drag-resize";
const resizableProps = {
  sticks: Array,
  x: Number,
  y: Number,
  w: Number,
  h: Number,
  initW: Number,
  initH: Number,
  minW: {
    type: Number,
    default: 20
  },
  minH: {
    type: Number,
    default: 20
  },
  maxW: {
    type: Number,
    default: void 0
  },
  maxH: {
    type: Number,
    default: void 0
  },
  active: {
    type: Boolean,
    default: void 0
  },
  type: {
    type: String,
    default: "default"
  },
  resizingClass: [String, Object, Array],
  resizableClass: [String, Object, Array],
  activeClass: [String, Object, Array],
  activeStyle: [String, Object, Array],
  stickClass: [String, Object, Array],
  stickStyle: [String, Object, Array],
  aspectRatio: Boolean,
  resizable: {
    type: Boolean,
    default: true
  },
  draggable: {
    type: Boolean,
    default: true
  },
  dragTarget: Object
};
const Resizable = vue.defineComponent({
  name: prefixCls,
  props: resizableProps,
  emits: ["update:x", "update:y", "update:w", "update:h", "update:active", "resizing", "resize-start", "resize-end", "activated", "deactivated"],
  setup(props, {
    slots,
    emit
  }) {
    const resizingRef = vue.ref(false);
    const xref = createBinding("x"), wref = createBinding("w", props.initW);
    const yref = createBinding("y"), href = createBinding("h", props.initH);
    const clazzRef = vue.computed(() => vue.normalizeClass([prefixCls, `${prefixCls}-${props.type}`, resizingRef.value && props.resizingClass, props.resizable && props.resizableClass, activeRef.value && [`${prefixCls}-active`, props.activeClass]]));
    const styleRef = vue.computed(() => {
      let style = {
        left: xref.value + "px",
        top: yref.value + "px",
        width: wref.value + "px",
        height: href.value + "px"
      };
      if (activeRef.value && props.activeStyle) {
        style = vue.normalizeStyle([props.activeStyle, style]);
      }
      return style;
    });
    function getPosition(e) {
      if ("touches" in e) {
        return [e.touches[0].pageX, e.touches[0].pageY];
      } else {
        return [e.pageX, e.pageY];
      }
    }
    function ensureInRange(num, min, max) {
      return Math.max(min, Math.min(max, num));
    }
    function createBinding(prop, initV) {
      const mergeRef = useMergedState(vue.toRef(props, prop), vue.ref(initV), () => emit(`update:${prop}`));
      return mergeRef;
    }
    const elRef = vue.ref();
    let startPageX = 0, startPageY = 0, startX = 0, startY = 0, startW = 0, startH = 0;
    let stick = "";
    let transformOrigin;
    let transformOriginMap = {
      tl: (x, y, w, h) => [w, h],
      tc: (x, y, w, h) => [w / 2, h],
      tr: (x, y, w, h) => [0, h],
      cl: (x, y, w, h) => [w, h / 2],
      cr: (x, y, w, h) => [0, h / 2],
      bl: (x, y, w, h) => [w, 0],
      bc: (x, y, w, h) => [w / 2, 0],
      br: (x, y, w, h) => [0, 0]
    };
    function onstickDown(e, stickType) {
      if (!props.resizable)
        return;
      e.preventDefault();
      e.stopPropagation();
      resizingRef.value = true;
      [startPageX, startPageY] = getPosition(e);
      startX = parseInt(elRef.value.style.left) || 0;
      startY = parseInt(elRef.value.style.top) || 0;
      startW = elRef.value.offsetWidth;
      startH = elRef.value.offsetHeight;
      transformOrigin = transformOriginMap[stickType](startX, startY, startW, startH);
      stick = stickType;
      window.addEventListener("mousemove", onMousemove);
      window.addEventListener("mouseup", onMouseup);
      emit("resize-start");
    }
    function onMousemove(e) {
      var _a, _b;
      e.preventDefault();
      const [x, y] = getPosition(e);
      let deltaX = x - startPageX, deltaY = y - startPageY;
      const maxW = (_a = props.maxW) != null ? _a : Infinity, minW = Math.max(props.minW, 0);
      const setW = (rate, syncH) => {
        rate = Math.max(rate, 0);
        let w = ensureInRange(startW * rate, minW, maxW);
        const minx = startX + (startW - maxW) * (transformOrigin[0] / startW);
        const maxx = startX + (startW - minW) * (transformOrigin[0] / startW);
        const x2 = ensureInRange(startX + transformOrigin[0] * (1 - rate), minx, maxx);
        xref.value = x2;
        wref.value = w;
        if (syncH)
          setH(rate);
      };
      const maxH = (_b = props.maxH) != null ? _b : Infinity, minH = Math.max(props.minH, 0);
      const setH = (rate, syncW) => {
        rate = Math.max(rate, 0);
        let h = ensureInRange(startH * rate, minH, maxH);
        const miny = startY + (startH - maxH) * (transformOrigin[1] / startH);
        const maxy = startY + (startH - minH) * (transformOrigin[1] / startH);
        const y2 = ensureInRange(startY + transformOrigin[1] * (1 - rate), miny, maxy);
        yref.value = y2;
        href.value = h;
        if (syncW)
          setW(rate);
      };
      if (stick.includes("l") || stick.includes("r")) {
        deltaX *= stick.includes("l") ? -1 : 1;
        let distance = stick.includes("l") ? transformOrigin[0] : startW - transformOrigin[0];
        let rate = (distance + deltaX) / distance;
        setW(rate, props.aspectRatio);
      }
      if (stick.includes("t") || stick.includes("b")) {
        deltaY *= stick.includes("t") ? -1 : 1;
        let distance = stick.includes("t") ? transformOrigin[1] : startH - transformOrigin[1];
        let rate = (distance + deltaY) / distance;
        setH(rate, props.aspectRatio);
      }
      emit("resizing");
    }
    function onMouseup() {
      window.removeEventListener("mousemove", onMousemove);
      window.removeEventListener("mouseup", onMouseup);
      emit("resize-end");
    }
    const {
      enable
    } = useDraggable(elRef, {
      dragTarget: props.dragTarget,
      x: xref,
      y: yref
    });
    vue.onMounted(() => {
      vue.watchPostEffect(() => enable(props.draggable));
    });
    vue.onUnmounted(() => {
      enable(false);
    });
    const activeRef = useMergedState(vue.toRef(props, "active"), vue.ref(false), setActive);
    function setActive(val) {
      emit(val ? "activated" : "deactivated");
      emit("update:active", val);
    }
    function onActive() {
      if (activeRef.value)
        return;
      activeRef.value = true;
    }
    function onDeactivated(e) {
      if (!activeRef.value)
        return;
      if (elRef.value.contains(e.target))
        return;
      activeRef.value = false;
    }
    vue.onMounted(() => {
      document.documentElement.addEventListener("mousedown", onDeactivated);
    });
    vue.onUnmounted(() => {
      document.documentElement.removeEventListener("mousedown", onDeactivated);
    });
    const stickNodes = vue.computed(() => (props.resizable ? props.sticks || sticks : []).map((item) => vue.createVNode("div", {
      "class": `${prefixCls}_stick ${prefixCls}_stick-${item} ${props.stickClass || ""} ${props.stickClass || ""}-${item}`,
      "style": props.stickStyle,
      "onMousedown": (e) => onstickDown(e, item),
      "onTouchstart": (e) => onstickDown(e, item)
    }, null)));
    return () => {
      var _a;
      return vue.createVNode("div", {
        "class": clazzRef.value,
        "style": styleRef.value,
        "ref": elRef,
        "onMousedown": onActive
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots), stickNodes.value]);
    };
  }
});
Resizable.install = (app) => {
  app.component(Resizable.name, Resizable);
};

exports["default"] = Resizable;
exports.resizableProps = resizableProps;
