'use strict';

var vue = require('vue');
var Schema = require('async-validator');
var _ = require('../_util/_.js');
var index = require('../_util/props-util/index.js');
var _interface = require('./interface.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Schema__default = /*#__PURE__*/_interopDefaultLegacy(Schema);

const formItemProps = {
  prefixCls: {
    default: "x-form-item"
  },
  layout: String,
  label: String,
  labelWidth: Number,
  prop: String,
  rules: [Object, Array],
  required: Boolean,
  first: Boolean,
  extra: String
};
var FormItem = vue.defineComponent({
  name: "x-form-item",
  props: formItemProps,
  setup(props, {
    expose,
    slots
  }) {
    const {
      proxy: vm
    } = vue.getCurrentInstance();
    const formContext = vue.inject(_interface.FormContextKey);
    const formProps = (formContext == null ? void 0 : formContext.props) || {};
    const formRulesRef = vue.computed(() => _.castArray(_.get(formProps.rules, props.prop) || []));
    const rulesRef = vue.computed(() => {
      const rules = formRulesRef.value.concat(props.rules || []);
      if (props.required)
        rules.push({
          required: true,
          trigger: "change",
          type: "any"
        });
      return rules;
    });
    const valueRef = vue.computed(() => _.get(formProps.model, props.prop));
    const initialValue = _.cloneDeep(valueRef.value);
    const labelWidthRef = vue.computed(() => {
      var _a, _b;
      return (_b = (_a = props.labelWidth) != null ? _a : formProps.labelWidth) != null ? _b : void 0;
    });
    const layoutRef = vue.computed(() => props.layout || formProps.layout);
    const requiredRef = vue.computed(() => rulesRef.value.some((rule) => rule.required));
    const explainsRef = vue.ref([]);
    const labelNodeRef = vue.computed(() => {
      const layout = layoutRef.value;
      const child = index.getComponent(vm, "label");
      if (layout === "vertical" && !child) {
        return void 0;
      }
      const {
        prefixCls
      } = props;
      const labelProps = {
        class: `${prefixCls}_label`,
        style: {
          width: labelWidthRef.value + "px"
        }
      };
      return vue.createVNode("label", labelProps, [index.getComponent(vm, "label")]);
    });
    const exposeProps = {
      props,
      valueRef,
      validate,
      resetField,
      clearValidate
    };
    if (formContext) {
      formContext.addItem(exposeProps);
      vue.onBeforeUnmount(() => {
        formContext.delItem(exposeProps);
      });
    }
    function validate(options = {}) {
      const {
        prop,
        first
      } = props;
      if (prop == null)
        return Promise.resolve();
      if (options.first === void 0)
        options.first = first;
      const trigger = options.trigger;
      let rules = rulesRef.value;
      if (trigger)
        rules = rules.filter((rule) => {
          var _a;
          return (_a = rule.trigger) == null ? void 0 : _a.includes(trigger);
        });
      if (!rules.length) {
        return Promise.resolve();
      }
      const validator = new Schema__default["default"]({
        [prop]: rules
      });
      return new Promise((resolve, reject) => {
        validator.validate({
          [prop]: valueRef.value
        }, options, (errors) => {
          if (errors == null ? void 0 : errors.length) {
            explainsRef.value = errors.map((e) => e.message);
            reject({
              errors
            });
          } else {
            clearValidate();
            resolve({});
          }
        });
      });
    }
    function resetField() {
      _.set(formProps.model, props.prop, _.cloneDeep(initialValue));
      clearValidate();
    }
    function clearValidate() {
      explainsRef.value = [];
    }
    expose(exposeProps);
    function onFocus() {
      validate({
        trigger: "focus"
      });
    }
    function onBlur() {
      validate({
        trigger: "blur"
      });
    }
    function onChange() {
      clearValidate();
      validate({
        trigger: "change"
      });
    }
    return () => {
      var _a;
      const {
        prefixCls
      } = props;
      const cls = [prefixCls, `${prefixCls}-${layoutRef.value}`, {
        [`${prefixCls}-error`]: explainsRef.value.length,
        [`${prefixCls}-required`]: requiredRef.value
      }];
      const children = index.flattenChildren((_a = slots.default) == null ? void 0 : _a.call(slots));
      let child = children[0];
      if (index.isValidElement(child)) {
        child = vue.cloneVNode(child, {
          onFocus,
          onBlur,
          onChange
        });
      }
      return vue.createVNode("div", {
        "class": cls
      }, [labelNodeRef.value, vue.createVNode("div", {
        "class": `${prefixCls}_wrapper`
      }, [vue.createVNode("div", {
        "class": `${prefixCls}_input`
      }, [[child, children.slice(1)]]), vue.createVNode("div", {
        "class": `${prefixCls}_feedback`
      }, [explainsRef.value.map((msg) => vue.createVNode("div", {
        "class": `${prefixCls}_feedback_line`
      }, [msg])), props.extra ? vue.createVNode("div", {
        "class": `${prefixCls}_extra`
      }, [index.getComponent(vm, "extra")]) : void 0])])]);
    };
  }
});

module.exports = FormItem;
