'use strict';

var vue = require('vue');
var shared = require('@vue/shared');
var _ = require('../_util/_.js');
var index = require('../_util/props-util/index.js');
require('../_util/hooks/index.js');
var isValid = require('../_util/isValid.js');
require('../icon/index.js');
var inputProps = require('./inputProps.js');
var utils = require('./utils.js');
var useNameSpace = require('../_util/hooks/useNameSpace2.js');
var useMemo = require('../_util/hooks/useMemo.js');
var Icon = require('../icon/Icon.js');

const prefixCls = "x-input";
var Input = vue.defineComponent({
  name: prefixCls,
  props: inputProps,
  emits: ["update:value", "change", "input", "pressEnter", "keydown", "focus", "blur"],
  setup(props, {
    emit,
    expose
  }) {
    const ns = useNameSpace(prefixCls);
    const vm = vue.getCurrentInstance().proxy;
    const inputRef = vue.ref();
    const focusedRef = vue.ref();
    const valRef = useMemo(() => props.pair ? _.castArray(props.value) : props.value);
    const showClearRef = vue.computed(() => _.castArray(valRef.value).some(isValid) && !props.disabled);
    function onInput(e) {
      var _a, _b;
      const value = (_b = (_a = e == null ? void 0 : e.target) == null ? void 0 : _a.value) != null ? _b : "";
      if (props.pair) {
        if (!shared.isArray(valRef.value)) {
          throw new Error("valRef.value is not a Array type");
        }
        if (e) {
          if (e.target == inputRef.value)
            valRef.value[0] = value;
          else
            valRef.value[1] = value;
        } else {
          valRef.value = ["", ""];
        }
      } else {
        valRef.value = value;
      }
      emit("update:value", valRef.value);
      emit("input", valRef.value);
    }
    function onChange(e) {
      emit("change", valRef.value);
    }
    function onFocus(e) {
      focusedRef.value = true;
      emit("focus", e);
    }
    function onBlur(e) {
      focusedRef.value = false;
      emit("blur", e);
    }
    function onKeydown(e) {
      if (e.keyCode === 13) {
        emit("pressEnter", e);
      }
      emit("keydown", e);
    }
    function onClearDown() {
      window.requestAnimationFrame(focus);
    }
    function onClear(e) {
      e.stopPropagation();
      inputRef.value.value = "";
      onInput();
      focus();
    }
    function focus() {
      inputRef.value.focus();
    }
    function blur() {
      inputRef.value.blur();
    }
    expose({
      focus,
      blur,
      get value() {
        return valRef.value;
      }
    });
    const on = {
      onInput,
      onChange,
      onKeydown,
      onFocus,
      onBlur
    };
    function renderInput() {
      const {
        disabled,
        type
      } = props;
      const placeholder = _.castArray(props.placeholder);
      const value = _.castArray(valRef.value);
      const addon1Node = index.getComponent(vm, "addonBefore");
      const addon2Node = index.getComponent(vm, "addonAfter");
      const prefixNode = index.getComponent(vm, "prefix");
      const suffixNode = index.getComponent(vm, "suffix");
      return vue.createVNode(vue.Fragment, null, [addon1Node && vue.createVNode("span", {
        "class": ns.e("addon")
      }, [addon1Node]), prefixNode && vue.createVNode("span", {
        "class": ns.e("prefix")
      }, [prefixNode]), vue.createVNode("input", vue.mergeProps({
        "class": ns.e("input"),
        "ref": inputRef,
        "value": value[0],
        "type": type,
        "disabled": disabled,
        "placeholder": placeholder[0]
      }, on), null), props.pair && vue.createVNode("span", {
        "class": ns.e("separator")
      }, [props.separator]), props.pair && vue.createVNode("input", vue.mergeProps({
        "class": ns.e("input"),
        "value": value[1],
        "type": type,
        "disabled": disabled,
        "placeholder": placeholder[1] || placeholder[0]
      }, on), null), suffixNode && vue.createVNode("span", {
        "class": ns.e("suffix")
      }, [suffixNode]), addon2Node && vue.createVNode("span", {
        "class": ns.e("addon")
      }, [addon2Node])]);
    }
    function renderTextarea() {
      const {
        disabled,
        placeholder
      } = props;
      const style = utils.calcTextareaHeight(inputRef.value, props.autosize);
      return vue.createVNode("textarea", vue.mergeProps({
        "class": ns.e("input"),
        "ref": inputRef,
        "style": style,
        "value": valRef.value,
        "disabled": disabled,
        "placeholder": placeholder
      }, on), null);
    }
    return () => {
      const {
        disabled,
        clearable
      } = props;
      const cls = [ns.b(), ns.m(props.type), props.pair && ns.m("pair"), disabled && ns.m("disabled"), focusedRef.value && ns.m(`focus`)];
      return vue.createVNode("div", {
        "class": cls
      }, [props.type !== "textarea" ? renderInput() : renderTextarea(), clearable && vue.withDirectives(vue.createVNode("span", {
        "class": `${prefixCls}_clear`,
        "onMousedown": onClearDown,
        "onClick": onClear
      }, [vue.createVNode(Icon["default"], {
        "type": "cross"
      }, null)]), [[vue.vShow, showClearRef.value]])]);
    };
  }
});

module.exports = Input;
