'use strict';

var vue = require('vue');
var core = require('@vueuse/core');
var index = require('../_util/props-util/index.js');
var utils = require('./utils.js');
var Marks = require('./Marks.js');
var Hanlde = require('./Hanlde.js');

const prefixCls = "x-slider";
const Slider = vue.defineComponent({
  name: prefixCls,
  props: {
    value: [Number, Array],
    disabled: Boolean,
    step: {
      default: 1
    },
    min: {
      default: 0
    },
    max: {
      default: 100
    },
    range: Boolean,
    marks: Object,
    formatTooltip: Function,
    tooltip: {
      default: true
    }
  },
  emits: ["update:value", "change"],
  setup(props, {
    attrs,
    emit
  }) {
    const {
      proxy: self
    } = vue.getCurrentInstance();
    const sliderRef = vue.ref();
    const _value = vue.computed(() => Array.isArray(props.value) ? props.value : [props.value || 0]);
    const state = vue.reactive({
      value: _value.value
    });
    vue.watch(_value, (val) => {
      if (val.toString() === state.value.toString())
        return;
      state.value = val;
    });
    function getSliderLength() {
      if (!sliderRef.value)
        return 0;
      return sliderRef.value.getBoundingClientRect().width;
    }
    function calcValue(offset) {
      const ratio = offset / getSliderLength();
      let v = props.min + (props.max - props.min) * ratio;
      return utils.ensureValueInRange(v, props);
    }
    function calcValueByOffset(offset) {
      return trimAlignValue(calcValue(offset));
    }
    function getClosestHandle(val) {
      const arr = state.value;
      const i = arr.findIndex((e) => val < e) - 1;
      if (i == -1)
        return 0;
      if (i == -2)
        return arr.length - 1;
      const a = val - arr[i], b = arr[i + 1] - val;
      if (a === b)
        return currHandle;
      return a < b ? i : i + 1;
    }
    function trimAlignValue(val) {
      val = utils.ensureValueInRange(val, props);
      return utils.ensureValuePrecision(val, props);
    }
    const handleRefs = [];
    function saveHanels(vm) {
      handleRefs.push(index.findDOMNode(vm));
    }
    function getHanleNode() {
      return state.value.map((e) => {
        const {
          min,
          max,
          disabled,
          formatTooltip,
          tooltip
        } = props;
        return vue.createVNode(Hanlde, vue.mergeProps({
          "value": e,
          "ref": saveHanels
        }, {
          prefixCls,
          min,
          max,
          disabled,
          formatTooltip,
          tooltip
        }, {
          "onMousedown": onMousedown,
          "onMouseup": onMouseup
        }), null);
      });
    }
    function getBarNode() {
      const {
        min,
        max
      } = props, len = max - min;
      let style = {};
      if (props.range) {
        style.left = `${(state.value[0] - min) / len * 100}%`;
        style.right = `${100 - (state.value[1] - min) / len * 100}%`;
      } else {
        style.left = `0`;
        style.right = `${100 - (state.value[0] - min) / len * 100}%`;
      }
      const barProps = {
        class: [`${prefixCls}_bar`],
        style
      };
      return vue.createVNode("div", barProps, null);
    }
    core.useEventListener(sliderRef, "mousedown", onMousedown);
    core.useEventListener(sliderRef, "touchstart", onMousedown);
    let startPos, startOffset, currHandle;
    function onClickLabel(e, value) {
      const handle = getClosestHandle(value);
      state.value[handle] = value;
      onChange(state.value);
    }
    function onMousedown(e) {
      var _a;
      e.preventDefault();
      e.stopPropagation();
      onMouseup();
      const pageX = "pageX" in e ? e.pageX : (_a = e.touches[0]) == null ? void 0 : _a.pageX;
      const target = e.target;
      currHandle = handleRefs.findIndex((e2) => e2 === target);
      if (currHandle > -1) {
        startOffset = target.offsetLeft;
        target.focus();
      } else {
        startOffset = pageX - sliderRef.value.getBoundingClientRect().left;
        const val = calcValueByOffset(startOffset);
        currHandle = getClosestHandle(val);
        state.value[currHandle] = val;
        onChange(state.value);
        handleRefs[currHandle].focus();
      }
      startPos = pageX;
      window.addEventListener("mousemove", onMousemove, {
        passive: true
      });
      window.addEventListener("mouseup", onMouseup, {
        passive: true
      });
      window.addEventListener("touchmove", onMousemove, {
        passive: true
      });
      window.addEventListener("touchend", onMouseup, {
        passive: true
      });
    }
    function onMousemove(e) {
      var _a;
      const pageX = "pageX" in e ? e.pageX : (_a = e.touches[0]) == null ? void 0 : _a.pageX;
      const offset = pageX - startPos + startOffset, i = currHandle;
      let v = calcValueByOffset(offset);
      if (v < state.value[currHandle - 1]) {
        [state.value[currHandle - 1], state.value[currHandle]] = [v, state.value[currHandle - 1]];
        currHandle--;
      } else if (v > state.value[currHandle + 1]) {
        [state.value[currHandle + 1], state.value[currHandle]] = [v, state.value[currHandle + 1]];
        currHandle++;
      } else {
        state.value[currHandle] = v;
      }
      onChange(state.value);
      if (i != currHandle) {
        Object.defineProperty(e, "target", {
          writable: true
        });
        e.target = handleRefs[currHandle];
        onMousedown(e);
      }
    }
    function onMouseup() {
      handleRefs.forEach((e) => e.blur());
      window.removeEventListener("mousemove", onMousemove);
      window.removeEventListener("mouseup", onMouseup);
      window.removeEventListener("touchmove", onMousemove);
      window.removeEventListener("touchend", onMouseup);
    }
    function onChange(value) {
      value = props.range ? value : value[0];
      emit("update:value", value);
      emit("change", value);
    }
    return () => {
      const sliderProps = {
        ref: sliderRef,
        class: [prefixCls, {
          [`${prefixCls}-marks`]: !!props.marks,
          [`${prefixCls}-disabled`]: props.disabled
        }]
      };
      return vue.createVNode("div", sliderProps, [getBarNode(), vue.createVNode(Marks, vue.mergeProps({
        "prefixCls": prefixCls
      }, props, {
        "onClickLabel": onClickLabel
      }), null), getHanleNode()]);
    };
  }
});
Slider.install = (app) => {
  app.component(Slider.name, Slider);
};

module.exports = Slider;
