'use strict';

var vue = require('vue');
var shared = require('@vue/shared');
var core = require('@vueuse/core');
var index = require('../_util/props-util/index.js');
var ensureRange = require('../_util/ensureRange.js');

function _isSlot(s) {
  return typeof s === "function" || Object.prototype.toString.call(s) === "[object Object]" && !vue.isVNode(s);
}
const prefixCls = "x-virtual-list";
const VirtualList = vue.defineComponent({
  name: prefixCls,
  props: {
    items: Array,
    itemKey: [String, Function],
    itemsWrap: [Object, Function],
    virtual: Boolean,
    bufferSize: {
      type: Number,
      default: 0
    },
    onScroll: Function
  },
  emits: ["scroll"],
  setup(props, {
    expose,
    emit
  }) {
    const firstNode = vue.ref();
    const ih = vue.ref(0);
    core.useResizeObserver(firstNode, ([entry]) => ih.value = entry.target.offsetHeight);
    const list = vue.computed(() => props.items || shared.EMPTY_ARR);
    const el = vue.ref();
    const h = core.useElementSize(el).height;
    const barRef = vue.ref();
    const state = vue.reactive({
      si: 0,
      ei: Math.min(1, list.value.length)
    });
    const showList = vue.computed(() => list.value.slice(state.si, state.ei));
    const sh = vue.computed(() => Math.max(list.value.length * ih.value, h.value));
    const st = vue.ref(0);
    function getIndex(index) {
      return state.si + index;
    }
    function getKey(item, index) {
      var _a;
      if (typeof props.itemKey === "function")
        return props.itemKey(item);
      return (_a = item[props.itemKey]) != null ? _a : index;
    }
    vue.watchEffect(() => {
      if (!props.virtual) {
        state.si = 0;
        state.ei = list.value.length;
        return;
      }
      const scrollTop = st.value;
      const si = ~~(scrollTop / ih.value);
      const ei = Math.ceil((scrollTop + h.value) / ih.value);
      state.si = Math.max(si - props.bufferSize, 0);
      state.ei = ih.value === 0 ? si + 1 : Math.min(ei + props.bufferSize, list.value.length);
    });
    function scroll(top) {
      top = ensureRange(top, 0, sh.value - h.value);
      st.value = el.value.scrollTop = top;
    }
    function onScroll(e) {
      const {
        scrollTop
      } = e.currentTarget;
      scroll(scrollTop);
      emit("scroll", e);
    }
    return (vm) => {
      const {
        virtual,
        itemsWrap
      } = props;
      const comStyle = virtual ? `overflow-y:overlay; overflow-anchor:none;` : "overflow:overlay;";
      const childRender = (item, index$1) => index.getSlot(vm, "default", {
        data: showList.value,
        item,
        index: getIndex(index$1)
      })[0];
      const itemsNode = showList.value.map((item, index) => vue.cloneVNode(childRender(item, index), {
        key: getKey(item, getIndex(index)),
        class: `${prefixCls}_item`,
        ref: index == 0 ? firstNode : void 0
      }));
      let wrapNode = itemsWrap ? vue.createVNode(itemsWrap, null, _isSlot(itemsNode) ? itemsNode : {
        default: () => [itemsNode]
      }) : vue.createVNode("div", {
        "style": "display:flex; flex-direction:column; flex-wrap:wrap;"
      }, [itemsNode]);
      wrapNode = vue.cloneVNode(wrapNode, {
        style: `padding-top:${state.si * ih.value}px; position:relative; ${props.virtual ? `height:${sh.value}px;` : ""} box-sizing:border-box;`
      });
      return vue.createVNode("div", {
        "ref": el,
        "class": `${prefixCls}_wrapper`,
        "style": comStyle,
        "onScroll": onScroll
      }, [wrapNode]);
    };
  }
});
VirtualList.install = (app) => {
  app.component(VirtualList.name, VirtualList);
};

module.exports = VirtualList;
