import { isVNode, defineComponent, ref, computed, nextTick, createVNode, mergeProps } from 'vue';
import { isArray } from '@vue/shared';
import moment from 'moment';
import useMemo from '../_util/hooks/useMemo.js';
import { castArray } from '../_util/_.js';
import { toMoment, formatDate } from './utils.js';
import { PickerProps } from './props.js';
import '../trigger/index.js';
import '../icon/index.js';
import '../input/index.js';
import '../calendar/index.js';
import RangeCalendar from '../calendar/RangeCalendar.js';
import { DateCalendar } from '../calendar/Calendar.js';
import Input from '../input/input.js';
import Icon from '../icon/Icon.js';
import Trigger from '../trigger/Trigger.js';

function _isSlot(s) {
  return typeof s === "function" || Object.prototype.toString.call(s) === "[object Object]" && !isVNode(s);
}
const prefixCls = "x-date-picker";
const DatePicker = defineComponent({
  name: prefixCls,
  props: PickerProps,
  emits: ["update:value", "change"],
  setup(props, {
    emit
  }) {
    const calendarRef = ref();
    const inputRef = ref();
    const visible = ref(false);
    const valueRef = useMemo(() => toMoment(props.value, props.valueFormat));
    const isRange = computed(() => props.type.includes("range"));
    function onChange(date) {
      valueRef.value = isRange.value ? castArray(date).map((e) => e == null ? void 0 : e.clone()) : date == null ? void 0 : date.clone();
      if (isArray(date)) {
        const val = props.valueFormat ? formatDate(date, props.valueFormat) : date == null ? void 0 : date.map((e) => e.clone());
        emit("update:value", val);
        emit("change", val);
      } else {
        const val = props.valueFormat ? formatDate(date, props.valueFormat) : date == null ? void 0 : date.clone();
        emit("update:value", val);
        emit("change", val);
      }
    }
    function onSelect(date) {
      console.log("date-picker");
      onChange(date);
      visible.value = false;
    }
    function onInput(str) {
      if (!str)
        return onChange(null);
      const date = toMoment(str, props.format, true);
      if (castArray(date).every((e) => e.isValid())) {
        onChange(date);
      }
    }
    function onKeydown(e) {
      if (e.key === "Enter")
        visible.value = !visible.value;
    }
    function onBlur() {
      var _a;
      const temp = valueRef.value;
      valueRef.value = ((_a = castArray(valueRef.value)[0]) != null ? _a : moment()).clone().add(1, "days");
      nextTick(() => valueRef.value = temp);
      const els = [calendarRef, inputRef].map((e) => e.value.$el).filter((e) => e);
      setTimeout(() => visible.value = els.some((e) => e == document.activeElement || e.contains(document.activeElement)), 0);
    }
    function renderCalendar() {
      const calendarProps = {
        ref: calendarRef,
        value: valueRef.value,
        disabledDate: props.disabledDate,
        tabindex: -1,
        onSelect,
        onBlur
      };
      return isRange.value ? createVNode(RangeCalendar, calendarProps, null) : createVNode(DateCalendar, calendarProps, null);
    }
    function renderInput() {
      const {
        clearable,
        disabled,
        placeholder
      } = props;
      const inputProps = {
        ref: inputRef,
        value: formatDate(valueRef.value, props.format),
        clearable,
        disabled,
        placeholder,
        pair: isRange.value,
        onInput,
        onKeydown
      };
      return createVNode(Input, mergeProps(inputProps, {
        "class": `${prefixCls}-range`
      }), {
        suffix: () => createVNode(Icon, {
          "type": "calendar"
        }, null)
      });
    }
    return () => {
      let _slot;
      const {
        disabled
      } = props;
      const triggerProps = {
        prefixCls: `${prefixCls}-popup`,
        popup: renderCalendar(),
        popupVisible: visible.value,
        action: disabled ? "" : "click",
        popupAlign: {
          points: ["tl", "bl"]
        },
        onFocus: () => visible.value = true,
        onBlur
      };
      return createVNode(Trigger, triggerProps, _isSlot(_slot = renderInput()) ? _slot : {
        default: () => [_slot]
      });
    };
  }
});
DatePicker.install = (app) => {
  app.component(DatePicker.name, DatePicker);
};

export { DatePicker as default };
