import { defineComponent, getCurrentInstance, inject, computed, ref, createVNode, onBeforeUnmount, cloneVNode } from 'vue';
import Schema from 'async-validator';
import { castArray, get, cloneDeep, set } from '../_util/_.js';
import { getComponent, flattenChildren, isValidElement } from '../_util/props-util/index.js';
import { FormContextKey } from './interface.js';

const formItemProps = {
  prefixCls: {
    default: "x-form-item"
  },
  layout: String,
  label: String,
  labelWidth: Number,
  prop: String,
  rules: [Object, Array],
  required: Boolean,
  first: Boolean,
  extra: String
};
var FormItem = defineComponent({
  name: "x-form-item",
  props: formItemProps,
  setup(props, {
    expose,
    slots
  }) {
    const {
      proxy: vm
    } = getCurrentInstance();
    const formContext = inject(FormContextKey);
    const formProps = (formContext == null ? void 0 : formContext.props) || {};
    const formRulesRef = computed(() => castArray(get(formProps.rules, props.prop) || []));
    const rulesRef = computed(() => {
      const rules = formRulesRef.value.concat(props.rules || []);
      if (props.required)
        rules.push({
          required: true,
          trigger: "change",
          type: "any"
        });
      return rules;
    });
    const valueRef = computed(() => get(formProps.model, props.prop));
    const initialValue = cloneDeep(valueRef.value);
    const labelWidthRef = computed(() => {
      var _a, _b;
      return (_b = (_a = props.labelWidth) != null ? _a : formProps.labelWidth) != null ? _b : void 0;
    });
    const layoutRef = computed(() => props.layout || formProps.layout);
    const requiredRef = computed(() => rulesRef.value.some((rule) => rule.required));
    const explainsRef = ref([]);
    const labelNodeRef = computed(() => {
      const layout = layoutRef.value;
      const child = getComponent(vm, "label");
      if (layout === "vertical" && !child) {
        return void 0;
      }
      const {
        prefixCls
      } = props;
      const labelProps = {
        class: `${prefixCls}_label`,
        style: {
          width: labelWidthRef.value + "px"
        }
      };
      return createVNode("label", labelProps, [getComponent(vm, "label")]);
    });
    const exposeProps = {
      props,
      valueRef,
      validate,
      resetField,
      clearValidate
    };
    if (formContext) {
      formContext.addItem(exposeProps);
      onBeforeUnmount(() => {
        formContext.delItem(exposeProps);
      });
    }
    function validate(options = {}) {
      const {
        prop,
        first
      } = props;
      if (prop == null)
        return Promise.resolve();
      if (options.first === void 0)
        options.first = first;
      const trigger = options.trigger;
      let rules = rulesRef.value;
      if (trigger)
        rules = rules.filter((rule) => {
          var _a;
          return (_a = rule.trigger) == null ? void 0 : _a.includes(trigger);
        });
      if (!rules.length) {
        return Promise.resolve();
      }
      const validator = new Schema({
        [prop]: rules
      });
      return new Promise((resolve, reject) => {
        validator.validate({
          [prop]: valueRef.value
        }, options, (errors) => {
          if (errors == null ? void 0 : errors.length) {
            explainsRef.value = errors.map((e) => e.message);
            reject({
              errors
            });
          } else {
            clearValidate();
            resolve({});
          }
        });
      });
    }
    function resetField() {
      set(formProps.model, props.prop, cloneDeep(initialValue));
      clearValidate();
    }
    function clearValidate() {
      explainsRef.value = [];
    }
    expose(exposeProps);
    function onFocus() {
      validate({
        trigger: "focus"
      });
    }
    function onBlur() {
      validate({
        trigger: "blur"
      });
    }
    function onChange() {
      clearValidate();
      validate({
        trigger: "change"
      });
    }
    return () => {
      var _a;
      const {
        prefixCls
      } = props;
      const cls = [prefixCls, `${prefixCls}-${layoutRef.value}`, {
        [`${prefixCls}-error`]: explainsRef.value.length,
        [`${prefixCls}-required`]: requiredRef.value
      }];
      const children = flattenChildren((_a = slots.default) == null ? void 0 : _a.call(slots));
      let child = children[0];
      if (isValidElement(child)) {
        child = cloneVNode(child, {
          onFocus,
          onBlur,
          onChange
        });
      }
      return createVNode("div", {
        "class": cls
      }, [labelNodeRef.value, createVNode("div", {
        "class": `${prefixCls}_wrapper`
      }, [createVNode("div", {
        "class": `${prefixCls}_input`
      }, [[child, children.slice(1)]]), createVNode("div", {
        "class": `${prefixCls}_feedback`
      }, [explainsRef.value.map((msg) => createVNode("div", {
        "class": `${prefixCls}_feedback_line`
      }, [msg])), props.extra ? createVNode("div", {
        "class": `${prefixCls}_extra`
      }, [getComponent(vm, "extra")]) : void 0])])]);
    };
  }
});

export { FormItem as default };
